@@ -131,12 +131,7 @@ void Pipeline::range_ranking()
 	for (PtrVector< ::Target>::iterator i = targets.begin(); i < targets.end();) {
 		Target* t = ((Target*)*i);
 		if (t->is_outranked(ip, source_query_len, rr)) {
-			if (config.benchmark_ranking) {
-				t->outranked = true;
-				++i;
-			}
-			else
-				i = targets.erase(i, i + 1);
+			i = targets.erase(i, i + 1);
 		}
 		else {
 			ip.insert(t->ungapped_query_range(source_query_len), t->filter_score);
@@ -175,7 +170,7 @@ void build_ranking_worker(PtrVector<::Target>::iterator begin, PtrVector<::Targe
 	}
 }
 
-void Pipeline::run(Statistics &stat, const sequence *subjects, size_t subject_count)
+void Pipeline::run(Statistics &stat)
 {
 	task_timer timer("Init banded swipe pipeline", target_parallel ? 3 : UINT_MAX);
 	Config::set_option(config.padding, 32);
