@@ -131,7 +131,6 @@ struct Config
 	int superblock;
 	unsigned max_cells;
 	int masking;
-	bool benchmark_ranking;
 	bool log_query;
 	bool log_subject;
 	unsigned threads_align;
@@ -239,6 +238,15 @@ struct Config
 	double ranking_score_drop_factor;
 	double ranking_cutoff_bitscore;
 	int left_most_interval;
+	bool alp;
+	bool no_forward_fp;
+	bool no_ref_masking;
+	string roc_file;
+	bool target_bias;
+	bool check_multi_target;
+	bool output_fp;
+	int family_cap;
+	int cbs_matrix_scale;
 
 	Sensitivity sensitivity;
 	TracebackMode traceback_mode;
@@ -250,7 +258,7 @@ struct Config
 		makedb = 0, blastp = 1, blastx = 2, view = 3, help = 4, version = 5, getseq = 6, benchmark = 7, random_seqs = 8, compare = 9, sort = 10, roc = 11, db_stat = 12, model_sim = 13,
 		match_file_stat = 14, model_seqs = 15, opt = 16, mask = 17, fastq2fasta = 18, dbinfo = 19, test_extra = 20, test_io = 21, db_annot_stats = 22, read_sim = 23, info = 24, seed_stat = 25,
 		smith_waterman = 26, cluster = 27, translate = 28, filter_blasttab = 29, show_cbs = 30, simulate_seqs = 31, split = 32, upgma = 33, upgma_mc = 34, regression_test = 35,
-		reverse_seqs = 36, compute_medoids = 37, mutate = 38, merge_tsv = 39
+		reverse_seqs = 36, compute_medoids = 37, mutate = 38, merge_tsv = 39, rocid = 40
 	};
 	unsigned	command;
 
