@@ -21,10 +21,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <algorithm>
 #include <stdexcept>
 #include <fstream>
+#include <vector>
+#include <sstream>
 #include "score_matrix.h"
-#include "../blast/raw_scoremat.h"
+#include "config.h"
 
 using std::string;
+using std::vector;
 
 Score_matrix score_matrix;
 
@@ -36,18 +39,6 @@ typedef double array_of_8[BLAST_NUM_STAT_VALUES];
 
 const double INT2_MAX = std::numeric_limits<double>::max();
 
-const signed char DNA_scores[5 * 5] = {
-	2, -3, -3, -3, -3,
-	-3,2,-3,-3,-3,
-	-3,-3,2,-3,-3,
-	-3,-3,-3,2,-3,
-	-3,-3,-3,-3,2
-};
-
-static array_of_8 dna_values[1] = {
-	{ 5, 2, (double)INT2_MAX, 0, 0, 0, 0, 0, 0, 0, 0 }
-};
-
 #define BLOSUM45_VALUES_MAX 14 /**< Number of different combinations supported for BLOSUM45. */
 static array_of_8 blosum45_values[BLOSUM45_VALUES_MAX] = {
 	{ (double)INT2_MAX, (double)INT2_MAX, (double)INT2_MAX, 0.2291, 0.0924, 0.2514, 0.9113, -5.7, 0.641318, 9.611060, 9.611060 },
@@ -175,7 +166,7 @@ struct Matrix_info
 {
 	const char *name;
 	const array_of_8 *constants;
-	const char *scores;
+	const signed char *scores;
 	const unsigned count;
 	const int default_gap_open, default_gap_extend;
 
@@ -197,50 +188,67 @@ struct Matrix_info
 		return 0;
 	}
 
-	static const Matrix_info matrices[9];
+	static const Matrix_info matrices[8];
 };
 
 const Matrix_info Matrix_info::matrices[] = {
-	{ "BLOSUM45", blosum45_values, (const char*)NCBISM_Blosum45.scores, BLOSUM45_VALUES_MAX, 14, 2 },
-	{ "BLOSUM50", blosum50_values, (const char*)NCBISM_Blosum50.scores, BLOSUM50_VALUES_MAX, 13, 2 },
-	{ "BLOSUM62", blosum62_values, (const char*)NCBISM_Blosum62.scores, BLOSUM62_VALUES_MAX, 11, 1 },
-	{ "BLOSUM80", blosum80_values, (const char*)NCBISM_Blosum80.scores, BLOSUM80_VALUES_MAX, 10, 1 },
-	{ "BLOSUM90", blosum90_values, (const char*)NCBISM_Blosum90.scores, BLOSUM90_VALUES_MAX, 10, 1 },
-	{ "PAM70", pam70_values, (const char*)NCBISM_Pam70.scores, PAM70_VALUES_MAX, 10, 1 },
-	{ "PAM250", pam250_values, (const char*)NCBISM_Pam250.scores, PAM250_VALUES_MAX, 14, 2 },
-	{ "PAM30", pam30_values, (const char*)NCBISM_Pam30.scores, PAM30_VALUES_MAX, 9, 1 },
-	{ "DNA", dna_values, (const char*)DNA_scores, 1, 4, 2 }
+	{ "BLOSUM45", blosum45_values, (const signed char*)s_Blosum45PSM, BLOSUM45_VALUES_MAX, 14, 2 },
+	{ "BLOSUM50", blosum50_values, (const signed char*)s_Blosum50PSM, BLOSUM50_VALUES_MAX, 13, 2 },
+	{ "BLOSUM62", blosum62_values, (const signed char*)s_Blosum62PSM, BLOSUM62_VALUES_MAX, 11, 1 },
+	{ "BLOSUM80", blosum80_values, (const signed char*)s_Blosum80PSM, BLOSUM80_VALUES_MAX, 10, 1 },
+	{ "BLOSUM90", blosum90_values, (const signed char*)s_Blosum90PSM, BLOSUM90_VALUES_MAX, 10, 1 },
+	{ "PAM70", pam70_values, (const signed char*)s_Pam70PSM, PAM70_VALUES_MAX, 10, 1 },
+	{ "PAM250", pam250_values, (const signed char*)s_Pam250PSM, PAM250_VALUES_MAX, 14, 2 },
+	{ "PAM30", pam30_values, (const signed char*)s_Pam30PSM, PAM30_VALUES_MAX, 9, 1 }
 };
 
-Score_matrix::Score_matrix(const string & matrix, int gap_open, int gap_extend, int frameshift, int stop_match_score, uint64_t db_letters):
+Score_matrix::Score_matrix(const string & matrix, int gap_open, int gap_extend, int frameshift, int stop_match_score, uint64_t db_letters, bool use_alp):
 	gap_open_ (gap_open == -1 ? Matrix_info::get(matrix).default_gap_open : gap_open),
 	gap_extend_ (gap_extend == -1 ? Matrix_info::get(matrix).default_gap_extend : gap_extend),
 	frame_shift_(frameshift),
 	db_letters_ ((double)db_letters),
 	constants_ (Matrix_info::get(matrix).get_constants(gap_open_, gap_extend_)),
+	ln_k_(log(k())),
 	name_(matrix),
 	matrix8_(Matrix_info::get(matrix).scores, stop_match_score),
 	bias_((char)(-low_score())),
 	matrix8u_(Matrix_info::get(matrix).scores, stop_match_score, bias_),
+	matrix8_low_(Matrix_info::get(matrix).scores, stop_match_score, 0, 16),
+	matrix8_high_(Matrix_info::get(matrix).scores, stop_match_score, 0, 16, 16),
+	matrix8u_low_(Matrix_info::get(matrix).scores, stop_match_score, bias_, 16),
+	matrix8u_high_(Matrix_info::get(matrix).scores, stop_match_score, bias_, 16, 16),
 	matrix16_(Matrix_info::get(matrix).scores, stop_match_score),
 	matrix32_(Matrix_info::get(matrix).scores, stop_match_score)
-{ }
+{ 
+	static const int N = 20;
+	if (use_alp) {
+		long m[N][N];
+		long* p[N];
+		for (int i = 0; i < N; ++i) {
+			for (int j = 0; j < N; ++j)
+				m[i][j] = (*this)(i, j);
+			p[i] = m[i];
+		}
+		evaluer.initGapped(N, p, background_freq, background_freq, gap_open_, gap_extend_, gap_open_, gap_extend_, false, 0.01, 0.05, 60.0, 1024.0, 0);
+		cout << evaluer.parameters().lambda << ' ' << evaluer.parameters().K << endl;
+	}
+}
 
-char Score_matrix::low_score() const
+int8_t Score_matrix::low_score() const
 {
-	char low = std::numeric_limits<char>::max();
-	for (Letter i = 0; i < (char)value_traits.alphabet_size; ++i)
-		for (Letter j = i + 1; j < (char)value_traits.alphabet_size; ++j)
-			low = std::min(low, (char)this->operator()(i, j));
+	int8_t low = std::numeric_limits<int8_t>::max();
+	for (Letter i = 0; i < (Letter)value_traits.alphabet_size; ++i)
+		for (Letter j = i + 1; j < (Letter)value_traits.alphabet_size; ++j)
+			low = std::min(low, (int8_t)this->operator()(i, j));
 	return low;
 }
 
-char Score_matrix::high_score() const
+int8_t Score_matrix::high_score() const
 {
-	char high = std::numeric_limits<char>::min();
-	for (Letter i = 0; i < (char)value_traits.alphabet_size; ++i)
-		for (Letter j = i; j < (char)value_traits.alphabet_size; ++j)
-			high = std::max(high, (char)this->operator()(i, j));
+	int8_t high = std::numeric_limits<int8_t>::min();
+	for (Letter i = 0; i < (Letter)value_traits.alphabet_size; ++i)
+		for (Letter j = i; j < (Letter)value_traits.alphabet_size; ++j)
+			high = std::max(high, (int8_t)this->operator()(i, j));
 	return high;
 }
 
@@ -262,14 +270,14 @@ std::ostream& operator<<(std::ostream& s, const Score_matrix &m)
 	return s;
 }
 
-const char* custom_scores(const string &matrix_file)
+const signed char* custom_scores(const string &matrix_file)
 {
-	static char scores[25 * 25];
+	static signed char scores[25 * 25];
 	string l, s;
 	std::stringstream ss;
 	vector<Letter> pos;
 	unsigned n = 0;
-	memset(scores, 0xff, sizeof(scores));
+	std::fill(scores, scores + sizeof(scores), '\xff');
 	if (matrix_file == "")
 		return scores;
 	std::ifstream f(matrix_file.c_str());
@@ -306,10 +314,15 @@ Score_matrix::Score_matrix(const string &matrix_file, double lambda, double K, i
 	gap_open_(gap_open),
 	gap_extend_(gap_extend),
 	db_letters_((double)db_letters),
+	ln_k_(log(K)),
 	name_("custom"),
 	matrix8_(custom_scores(matrix_file)),
 	bias_((char)(-low_score())),
 	matrix8u_(custom_scores(matrix_file), bias_),
+	matrix8_low_(custom_scores(matrix_file), bias_),
+	matrix8_high_(custom_scores(matrix_file), bias_),
+	matrix8u_low_(custom_scores(matrix_file), bias_),
+	matrix8u_high_(custom_scores(matrix_file), bias_),
 	matrix16_(custom_scores(matrix_file)),
 	matrix32_(custom_scores(matrix_file))
 {
