@@ -27,6 +27,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <stdint.h>
 #include "../util/log_stream.h"
 #include "value.h"
+#include "../lib/alp/sls_alignment_evaluer.hpp"
 
 using std::string;
 using std::cout;
@@ -38,7 +39,7 @@ struct Score_matrix
 {
 
 	Score_matrix() :ln_k_(0.0) {}
-	Score_matrix(const string &matrix, int gap_open, int gap_extend, int frame_shift, int stop_match_score, uint64_t db_letters = 0);
+	Score_matrix(const string &matrix, int gap_open, int gap_extend, int frame_shift, int stop_match_score, uint64_t db_letters = 0, bool use_alp=false);
 	Score_matrix(const string &matrix_file, double lambda, double K, int gap_open, int gap_extend, uint64_t db_letters = 0);
 
 	friend std::ostream& operator<<(std::ostream& s, const Score_matrix &m);
@@ -197,6 +198,7 @@ private:
 	Scores<int8_t> matrix8u_high_;
 	Scores<int16_t> matrix16_;
 	Scores<int> matrix32_;
+	Sls::AlignmentEvaluer evaluer;
 
 };
 
