@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2019 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,12 +20,12 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef SCORE_VECTOR_H_
-#define SCORE_VECTOR_H_
-
+#pragma once
+#include <algorithm>
 #include <limits.h>
 #include "../util/simd.h"
 #include "../basic/score_matrix.h"
+#include "../util/simd/vector.h"
 
 template<typename _score>
 struct score_traits
@@ -39,10 +43,24 @@ struct score_traits<uint8_t>
 	typedef uint16_t Mask;
 };
 
+namespace DISPATCH_ARCH {
+
+template<typename _t>
+struct ScoreTraits
+{
+	typedef void Score;
+	enum { CHANNELS = 0 };
+};
+
+
 template<typename _score>
 struct score_vector
 { };
 
+}
+
+namespace DISPATCH_ARCH {
+
 #ifdef __SSE2__
 
 template<>
@@ -58,8 +76,8 @@ struct score_vector<uint8_t>
 		data_ = _mm_setzero_si128();
 	}
 
-	explicit score_vector(char x):
-		data_ (_mm_set(x))
+	explicit score_vector(uint8_t x):
+		data_ (::SIMD::_mm_set1_epi8((char)x))
 	{ }
 
 	explicit score_vector(__m128i data):
@@ -92,12 +110,12 @@ struct score_vector<uint8_t>
 #ifdef __SSSE3__
 		const __m128i *row = reinterpret_cast<const __m128i*>(&score_matrix.matrix8u()[a << 5]);
 
-		__m128i high_mask = _mm_slli_epi16(_mm_and_si128(seq, _mm_set1_epi8('\x10')), 3);
+		__m128i high_mask = _mm_slli_epi16(_mm_and_si128(seq, ::SIMD::_mm_set1_epi8('\x10')), 3);
 		__m128i seq_low = _mm_or_si128(seq, high_mask);
-		__m128i seq_high = _mm_or_si128(seq, _mm_xor_si128(high_mask, _mm_set1_epi8('\x80')));
+		__m128i seq_high = _mm_or_si128(seq, _mm_xor_si128(high_mask, ::SIMD::_mm_set1_epi8('\x80')));
 
-		__m128i r1 = _mm_load_si128(row);
-		__m128i r2 = _mm_load_si128(row+1);
+		__m128i r1 = _mm_loadu_si128(row);
+		__m128i r2 = _mm_loadu_si128(row+1);
 		__m128i s1 = _mm_shuffle_epi8(r1, seq_low);
 		__m128i s2 = _mm_shuffle_epi8(r2, seq_high);
 		data_ = _mm_or_si128(s1, s2);
@@ -107,10 +125,11 @@ struct score_vector<uint8_t>
 	void set_generic(unsigned a, const __m128i &seq)
 	{
 		const uint8_t* row (&score_matrix.matrix8u()[a<<5]);
-		const uint8_t* seq_ptr (reinterpret_cast<const uint8_t*>(&seq));
-		uint8_t* dest (reinterpret_cast<uint8_t*>(&data_));
-		for(unsigned i=0;i<16;i++)
-			*(dest++) = row[*(seq_ptr++)];
+		uint8_t l[16], dest[16];
+		_mm_storeu_si128((__m128i*)l, seq);
+		for (unsigned i = 0; i < 16; i++)
+			dest[i] = row[(unsigned long)l[i]];
+		data_ = _mm_loadu_si128((const __m128i*)dest);
 	}
 
 	score_vector(const uint8_t* s):
@@ -133,9 +152,8 @@ struct score_vector<uint8_t>
 		return *this;
 	}
 
-	score_vector& operator++()
-	{
-		data_ = _mm_adds_epu8(data_, _mm_set(1));
+	score_vector& operator &=(const score_vector& rhs) {
+		data_ = _mm_and_si128(data_, rhs.data_);
 		return *this;
 	}
 
@@ -149,12 +167,17 @@ struct score_vector<uint8_t>
 
 	int operator [](unsigned i) const
 	{
-		return *(((uint8_t*)&data_)+i);
+		uint8_t s[16];
+		_mm_storeu_si128((__m128i*)s, data_);
+		return (int)s[i];
 	}
 
 	void set(unsigned i, uint8_t v)
 	{
-		*(((uint8_t*)&data_)+i) = v;
+		uint8_t s[16];
+		_mm_storeu_si128((__m128i*)s, data_);
+		s[i] = v;
+		data_ = _mm_loadu_si128((const __m128i*)s);
 	}
 
 	score_vector& max(const score_vector &rhs)
@@ -224,18 +247,24 @@ struct score_vector<uint8_t>
 
 #endif
 
-template<typename _t>
-struct ScoreTraits
-{
-	typedef void Score;
-	enum { CHANNELS = 0 };
-};
-
 template<>
 struct ScoreTraits<int32_t>
 {
-	enum { CHANNELS = 1 };
+	enum { CHANNELS = 1, BITS = 32 };
 	typedef int32_t Score;
+	struct TraceMask {
+		static uint8_t make(int vmask, int hmask) {
+			return vmask << 1 | hmask;
+		}
+		static uint8_t vmask(int channel) {
+			return 2;
+		}
+		static uint8_t hmask(int channel) {
+			return 1;
+		}
+		uint8_t gap;
+		uint8_t open;
+	};
 	static int32_t zero()
 	{
 		return 0;
@@ -256,15 +285,12 @@ struct ScoreTraits<int32_t>
 	{
 		return INT_MAX;
 	}
+	static constexpr int max_int_score() {
+		return INT_MAX;
+	}
 };
 
-inline void store_sv(int32_t sv, int32_t *dst)
-{
-	*dst = sv;
-}
-
 #ifdef __SSE2__
-
 template<>
 struct ScoreTraits<score_vector<uint8_t>>
 {
@@ -276,17 +302,35 @@ struct ScoreTraits<score_vector<uint8_t>>
 		return std::numeric_limits<uint8_t>::max();
 	}
 };
+#endif
+
+}
+
+static inline void store_sv(int32_t sv, int32_t *dst)
+{
+	*dst = sv;
+}
+
+static inline int32_t load_sv(const int32_t *x) {
+	return *x;
+}
+
+#ifdef __SSE2__
 
 template<typename _t, typename _p>
-inline void store_sv(const score_vector<_t> &sv, _p *dst)
+static inline void store_sv(const DISPATCH_ARCH::score_vector<_t> &sv, _p *dst)
 {
+#if ARCH_ID == 2
+	_mm256_storeu_si256((__m256i*)dst, sv.data_);
+#else
 	_mm_storeu_si128((__m128i*)dst, sv.data_);
+#endif
 }
 
 #endif
 
 template<typename _sv>
-inline typename ScoreTraits<_sv>::Score extract_channel(const _sv &v, int i) {
+static inline typename DISPATCH_ARCH::ScoreTraits<_sv>::Score extract_channel(const _sv &v, int i) {
 	return v[i];
 }
 
@@ -296,13 +340,11 @@ inline int extract_channel<int>(const int &v, int i) {
 }
 
 template<typename _sv>
-inline void set_channel(_sv &v, int i, typename ScoreTraits<_sv>::Score x) {
+static inline void set_channel(_sv &v, int i, typename DISPATCH_ARCH::ScoreTraits<_sv>::Score x) {
 	v.set(i, x);
 }
 
 template<>
 inline void set_channel<int>(int &v, int i, int x) {
 	v = x;
-}
-
-#endif /* SCORE_VECTOR_H_ */
+}
\ No newline at end of file
