@@ -22,12 +22,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #pragma once
 #include <assert.h>
 #include <limits>
+#include <vector>
 #include "../score_vector.h"
 #include "../score_vector_int8.h"
 #include "../score_vector_int16.h"
 #include "../../basic/value.h"
 #include "../util/simd/vector.h"
 #include "../../util/system.h"
+#include "../../util/memory/alignment.h"
+#ifdef __AVX2__
+#include "../../util/simd/transpose32x32.h"
+#endif
 
 static inline uint8_t cmp_mask(int x, int y) {
 	return x == y;
@@ -86,6 +91,16 @@ MSC_INLINE _sv add_cbs(const _sv& v, const _sv& query_bias) {
 	return v + query_bias;
 }
 
+template<typename _score>
+_score add_cbs_scalar(_score x, int8_t b) {
+	return x + _score(b);
+}
+
+template<typename _score>
+_score add_cbs_scalar(_score x, void *b) {
+	return x;
+}
+
 template<typename _sv>
 MSC_INLINE void make_gap_mask(typename ::DISPATCH_ARCH::ScoreTraits<_sv>::TraceMask *trace_mask, const _sv& current_cell, const _sv& vertical_gap, const _sv& horizontal_gap) {
 	trace_mask->gap = ::DISPATCH_ARCH::ScoreTraits<_sv>::TraceMask::make(cmp_mask(current_cell, vertical_gap), cmp_mask(current_cell, horizontal_gap));
@@ -104,6 +119,32 @@ template<typename _sv>
 MSC_INLINE void make_open_mask(std::nullptr_t, const _sv&, const _sv&, const _sv&) {
 }
 
+namespace DP {
+	struct NoCBS;
+}
+
+template<typename _sv, typename _cbs>
+struct CBSBuffer {
+	CBSBuffer(const DP::NoCBS&, int) {}
+	void* operator()(int i) const {
+		return nullptr;
+	}
+};
+
+template<typename _sv>
+struct CBSBuffer<_sv, const int8_t*> {
+	CBSBuffer(const int8_t* v, int l) {
+		data.reserve(l);
+		for (int i = 0; i < l; ++i)
+			data.emplace_back(typename ::DISPATCH_ARCH::ScoreTraits<_sv>::Score(v[i]));
+	}
+	_sv operator()(int i) const {
+		return data[i];
+	}
+	std::vector<_sv, Util::Memory::AlignmentAllocator<_sv, 32>> data;
+};
+
+
 template<typename _sv, typename _cbs, typename _trace_mask, typename _row_counter>
 MSC_INLINE _sv swipe_cell_update(const _sv &diagonal_cell,
 	const _sv &scores,
@@ -143,34 +184,6 @@ MSC_INLINE _sv swipe_cell_update(const _sv &diagonal_cell,
 }
 
 /*template<typename _sv>
-static inline _sv swipe_cell_update(const _sv &diagonal_cell,
-	const _sv &scores,
-	void*,
-	const _sv &gap_extension,
-	const _sv &gap_open,
-	_sv &horizontal_gap,
-	_sv &vertical_gap,
-	_sv &best,
-	TraceStat<_sv> &trace_stat_diag,
-	TraceStat<_sv> &trace_stat_vertical,
-	TraceStat<_sv> &trace_stat_horizontal,
-	void*,
-	const RowCounter<_sv>&)
-{
-	using std::max;
-	_sv current_cell = max(diagonal_cell + scores, vertical_gap);
-	current_cell = max(current_cell, horizontal_gap);
-	::DISPATCH_ARCH::ScoreTraits<_sv>::saturate(current_cell);
-	best = max(best, current_cell);
-	vertical_gap -= gap_extension;
-	horizontal_gap -= gap_extension;
-	const _sv open = current_cell - gap_open;
-	vertical_gap = max(vertical_gap, open);
-	horizontal_gap = max(horizontal_gap, open);
-	return current_cell;
-}
-
-template<typename _sv>
 static inline _sv swipe_cell_update(const _sv& diagonal_cell,
 	const _sv& scores,
 	const _sv& query_bias,
@@ -211,49 +224,6 @@ static inline _sv swipe_cell_update(const _sv& diagonal_cell,
 	return current_cell;
 }*/
 
-template<typename _sv>
-static inline _sv cell_update(const _sv &diagonal_cell,
-	const _sv &scores,
-	const _sv &gap_extension,
-	const _sv &gap_open,
-	_sv &horizontal_gap,
-	_sv &vertical_gap,
-	_sv &best,
-	const _sv &vbias)
-{
-	_sv current_cell = diagonal_cell + scores;
-	current_cell -= vbias;
-	current_cell.max(vertical_gap).max(horizontal_gap);
-	best.max(current_cell);
-	vertical_gap -= gap_extension;
-	horizontal_gap -= gap_extension;
-	const _sv open = current_cell - gap_open;
-	vertical_gap.max(open);
-	horizontal_gap.max(open);
-	return current_cell;
-}
-
-template<typename _score>
-static inline DISPATCH_ARCH::score_vector<_score> cell_update(const DISPATCH_ARCH::score_vector<_score> &diagonal_cell,
-	const DISPATCH_ARCH::score_vector<_score> &scores,
-	const DISPATCH_ARCH::score_vector<_score> &gap_extension,
-	const DISPATCH_ARCH::score_vector<_score> &gap_open,
-	DISPATCH_ARCH::score_vector<_score> &horizontal_gap,
-	DISPATCH_ARCH::score_vector<_score> &vertical_gap,
-	DISPATCH_ARCH::score_vector<_score> &best)
-{
-	using namespace ::DISPATCH_ARCH;
-	score_vector<_score> current_cell = diagonal_cell + scores;
-	current_cell.max(vertical_gap).max(horizontal_gap);
-	best.max(current_cell);
-	vertical_gap -= gap_extension;
-	horizontal_gap -= gap_extension;
-	const score_vector<_score> open = current_cell - gap_open;
-	vertical_gap.max(open);
-	horizontal_gap.max(open);
-	return current_cell;
-}
-
 template<typename _sv>
 static inline _sv cell_update(const _sv &diagonal_cell,
 	const _sv &shift_cell0,
@@ -303,11 +273,24 @@ struct SwipeProfile
 			data_[j] = _sv(j, seq);
 	}
 #endif
+
 	inline const _sv& get(Letter i) const
 	{
 		return data_[(int)i];
 	}
-	_sv data_[AMINO_ACID_COUNT];
+
+	void set(const int8_t** target_scores) {
+#if ARCH_ID == 2
+		transpose(target_scores, 32, (int8_t*)data_, __m256i());
+		for (size_t i = 0; i < AMINO_ACID_COUNT; ++i) {
+			data_[i].expand_from_8bit();
+		}
+#endif
+	}
+
+	//_sv data_[AMINO_ACID_COUNT];
+	_sv data_[32];
+
 };
 
 template<>
@@ -318,7 +301,8 @@ struct SwipeProfile<int32_t>
 	{
 		int16_t s[32];
 		_mm256_storeu_si256((__m256i*)s, seq);
-		row = score_matrix.row((char)s[0]);
+		const int* row = score_matrix.row((char)s[0]);
+		std::copy(row, row + 32, this->row);
 	}
 #endif
 #ifdef __SSE2__
@@ -326,18 +310,24 @@ struct SwipeProfile<int32_t>
 	{
 		int16_t s[8];
 		_mm_storeu_si128((__m128i*)s, seq);
-		row = score_matrix.row((char)s[0]);
+		const int* row = score_matrix.row((char)s[0]);
+		std::copy(row, row + 32, this->row);
 	}
 #endif
 	void set(uint64_t seq)
 	{
-		row = score_matrix.row((char)seq);
+		const int* row = score_matrix.row((char)seq);
+		std::copy(row, row + 32, this->row);
+	}
+	void set(const int8_t** target_scores) {
+		for (int i = 0; i < 32; ++i)
+			row[i] = target_scores[0][i];
 	}
 	int32_t get(char i) const
 	{
 		return row[(int)i];
 	}
-	const int32_t *row;
+	int32_t row[32];
 };
 
 }
